//
// Copyright (C) 2004  Cognos Incorporated.   All Rights Reserved.
//
// general (client and server) include file for planning70.asp, planning77.js
//
// This script is included into both server and client side pages, and so must not make use of features
// specific to one environment or another
//


// -<>---------------------------------------------------------------------------------------------
// Function: Error handling
// Purpose: 
// Comment:
// -><---------------------------------------------------------------------------------------------



// -<>---------------------------------------------------------------------------------------------
// Function: assignApplicationError
// Purpose:  direct assignment to error object properties, assumes all is valid.  
// Comment:  Originally only intended to be called from throwOrAssignApplicationError
// -><---------------------------------------------------------------------------------------------

function assignApplicationError(e, facility, componentErrorNumber, componentErrorDescription, applicationErrorSource, applicationErrorId, applicationErrorParameters)
{
    try
    {
        e.applicationPhaseId='';
        e.number = componentErrorNumber;
        e.description = componentErrorDescription;
        e.applicationErrorSource=applicationErrorSource;
        e.applicationErrorId=applicationErrorId;
        if (typeof(applicationErrorParameters)=='object')
        {
            e.applicationErrorParameters= applicationErrorParameters;   
        }
        else
        {
            e.applicationErrorParameters= new Array;       
        }
        e.allOtherErrors = new Array;                 // if more errors than the first, keep track of them here
        e.thisPageErrorDescription='';
        e.facility = facility;                        // 0 - client, 1 - web server, 2 - application server
        e.browserKnownGood=true;
    }
    catch(e)
    {
        e.applicationErrorSource='planning11.js::assignApplicationError';
        e.applicationErrorId='errorNestedErrorCatch';
        e.applicationErrorParameters= new Array;
        e.allOtherErrors = new Array;
        e.thisPageErrorDescription='';
        e.facility=0;
        e.browserKnownGood=false;
    }
}



// -<>---------------------------------------------------------------------------------------------
// Function: vThrowOrSetApplicationError
// Purpose:  Validate error parameters, assign to e if not null, else create and throw error
// Comment:  Check made on creation of the error here, and in the write response.       
//
// e may be null in which case this function should throw an error, or it may be an exisiting error
// to which this handler should assign information from the other arguments
//
// eCaught may be an unexpected error caught by the caller, and then passed to this function. It
// may also be an existing error being passed up the catch handler chain
// -><---------------------------------------------------------------------------------------------

function vThrowOrAssignApplicationError ( e, eCaught, applicationErrorSource, applicationErrorId, applicationErrorParameters)
{   

    var throwIt = false;

    try
    {
        webLogMessage(logLevelVerbose, 'vThrowOrAssignApplicationError: arguments.length=' + arguments.length);

        // check arguments, and make sure acceptable defaults are set if necessary
  
        if (arguments.length!=5)
        {
            webLogMessage(logLevelInformation, 'vThrowOrAssignApplicationError: arguments.length!=5: internal error');

            var internalError = new Error();
            assignApplicationError(internalError, 0, 0, 'vThrowOrAssignApplicationError expects 5 arguments, received ' + arguments.length, 'planning11.js::throwOrAssignApplicationError', 'errorMissingArguments');
	    throw internalError;
 	}

	if (typeof(e)!='object')                                    // typeof(null)=='object'
        {
            webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: arguments \'e\' not object');

            var internalError = new Error();
            assignApplicationError(internalError, 0, 0, 'throwOrAssignApplicationError \'e\' not object', 'planning11.js::throwOrAssignApplicationError', 'errorErrorNotObject');
	    throw internalError;
        }

        if (typeof(eCaught)!='object')
        {
            webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: \'eCaught\' not object: internal error');

            var internalError = new Error();
            assignApplicationError(internalError, 0, 0, 'throwOrAssignApplicationError \'rawError\' not object',
                                   'planning11.js::throwOrAssignApplicationError', 'errorErrorNotObject');
	    throw internalError;
        }

        if (eCaught)                              // eCaught can be null
        {      
            if (typeof(eCaught.number)!='number')  
            {
                webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: \'eCaught.number\' incorrect type: internal error');
 
                var internalError = new Error();
                assignApplicationError(internalError, 0, 0, 'throwOrAssignApplicationError \'eCaught.number\' incorrect type',
                                       'planning11.js::throwOrAssignApplicationError', 'errorValueTypeUnexpected', typeof(eCaught.number), 'number');
            }

            if (typeof(eCaught.description)!='string')  
            {
                webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: \'eCaught.description\' incorrect type: internal error');

                var internalError = new Error();
                assignApplicationError(internalError, 0, 0, 'throwOrAssignApplicationError \'eCaught.description\' incorrect type',
                                       'planning11.js::throwOrAssignApplicationError', 'errorValueTypeUnexpected', typeof(eCaught.description), 'string');
            }
        }


        if (typeof(applicationErrorSource)!='string') 
        {
            applicationErrorSource = 'planning11.js::throwOrAssignApplicationError';
        }
      
        if (typeof(applicationErrorId)!='string')
        {
            applicationErrorId = 'errorErrorIdNotSpecified';
        }     


        // eCaught is null, or an Error object with number and description properties 

        var componentErrorNumber =0;
        var componentErrorDescription = '';
        var eCaughtIsExisting = false;                 // rawError or fullError

        if (eCaught)
        {
            componentErrorNumber = eCaught.number;
            componentErrorDescription = eCaught.description;
 
            if (eCaught && (typeof(eCaught.applicationErrorId)=='string') )
            {
                eCaughtIsExisting = true;
            }  
        }

        // conditions                                        required action
        //
        // 1) e is null         , eCaught is null     : e = new error from args, default description and number, throw e
        // 2) e is null         , eCaught is rawError : e = new error from args, with rawError description and number, throw e
        // 3) e is null         , eCaught is existing : e = eCaught, new error from other args and add to error stack, throw e
        // 4) e in caller is raw, eCaught is null     : e = new error from args
        // 5) e in caller is raw, eCaught is rawError : e = new error from args (this one does not happen)..should really add rawError to stack
        // 6) e in caller is raw, eCaught is existing : copy eCaught to e, new error from other args and add to error stack
        // 7) e in caller exists, eCaught is null     : add new error from args to error stack
        // 8) e in caller exists, eCaught is rawError : add new error from args to error stack
        // 9) e is caller exists, eCaught is existing : add eCaught to e, new error from other args and add to error stack 
     
 
        // by extracting componentErrorNumber and componentErrorDescription above
        // condition 1==2, and 7==8

        var condition = 0;
              
        if (!e)
        {
            e = new Error;
            throwIt = true;

            if (eCaughtIsExisting)
            {
                condition= 3;
            }
            else
            { 
               if (eCaught) condition= 2; else condition =1;
            }
        }
        else // 5,6,8,9
        if (typeof(e.applicationErrorId)=='undefined')                           // e allocated in caller is raw
        {

            if (eCaughtIsExisting)
            {
		condition = 6;
            } 
            else
            {
		if (eCaught) condition= 5; else condition =4;
            }
        }
        else
        {
            if (eCaughtIsExisting)
            {
		condition = 9;
            } 
            else
            {
		if (eCaught) condition= 8; else condition =7;
            }
        }


	var eAssigned = null;

        webLogMessage(logLevelInformation, '...throwOrAssignApplicationError:condition=' + condition);
        webLogMessage(logLevelInformation, '...throwOrAssignApplicationError:componentErrorDescription=' + componentErrorDescription);

        switch(condition)
        {
            case 1:
            case 2:   // e =new error from args
            case 4:
            case 5:
                assignApplicationError(e, 0, componentErrorNumber, componentErrorDescription, applicationErrorSource, applicationErrorId, applicationErrorParameters);
                break; 

            case 3:   // e = eCaught, new error from other args and add to error stack, throw e
                e = eCaught;
                eAssigned = new Error(0, '');
                assignApplicationError(eAssigned, 0, componentErrorNumber, componentErrorDescription, applicationErrorSource, applicationErrorId, applicationErrorParameters);
                break; 

            case 6:    // e in caller is raw, eCaught is existing
                       // copy eCaught to e, new error from e and other args and add to error stack
                
                componentErrorNumber = e.number;
                componentErrorDescription = e.description;

                assignApplicationError(e, eCaught.serverReturnedError, eCaught.number, eCaught.description, eCaught.applicationErrorSource, eCaught.applicationErrorId, eCaught.applicationErrorParameters);
                e.allOtherErrors = eCaught.allOtherErrors;
 
                eAssigned = new Error(0, '');
                assignApplicationError(eAssigned, 0, componentErrorNumber, componentErrorDescription, applicationErrorSource, applicationErrorId, applicationErrorParameters);
                break;

             case 7:   // e in caller exists, eCaught is null
             case 8:   // add new error from args to error stack

                eAssigned = new Error(0, '');
                assignApplicationError(eAssigned, 0, componentErrorNumber, componentErrorDescription, applicationErrorSource, applicationErrorId, applicationErrorParameters);
                break; 

            case 9:   // e is caller exists, eCaught is existing
                      // add eCaught to e, new error from other args and add to error stack
                      // to do: (possibly) copy the allOtherErrors of eCaught to the allOtherErrors of e
                      // array.merge
                var znOtherErrorsSoFar =e.allOtherErrors.length;
                e.allOtherErrors[znOtherErrorsSoFar]= eCaught;

                eAssigned = new Error(0, '');
                assignApplicationError(eAssigned, 0, 0, '', applicationErrorSource, applicationErrorId, applicationErrorParameters);
                break;
        }

        if (eAssigned)   // conditions 3 (after e refers to eCaught) ,6 (after e is assigned), 7,8,9
        {
            var nOtherErrorsSoFar =e.allOtherErrors.length;
            e.allOtherErrors[nOtherErrorsSoFar]= eAssigned;
        }
    }
    catch(eNested)
    {

        // this means that there is some sort of error in the application error creation. 
        // try to do something sensible but not too complicated...don't try to keep previous errors
	var err = new Error(0, '');
        var componentErrorDescription ='nested error';
        var componentErrorNumber = 0;	   
   
        if (typeof(eNested)=='object')
        {
            if( typeof(eNested.number)=='string' || typeof(eNested.number)=='number')
            {
                componentErrorNumber = eNested.number;
            }

            if( typeof(eNested.description)=='string')
            {
                componentErrorDescription = eNested.description;
            }

        }
	else
        if (typeof(eNested)=='number')
        {    
            err.number = eNested;
        }

        assignApplicationError(err, 0, componentErrorNumber, componentErrorDescription,
                               'planningll.js::throwOrAssignApplicationError:catch', 'errorNestedError');
        
        throw err;
    }

    if (throwIt)
    {
        webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: throw error');

        throw e;
    }
    webLogMessage(logLevelInformation, '...throwOrAssignApplicationError normal return');

    return;
}


// -<>---------------------------------------------------------------------------------------------
// Function: throwOrAssignApplicationError
// Purpose:  
// Comment:  Convert any variable arguments into an array, and call vThrowOrSetApplicationError
// -><---------------------------------------------------------------------------------------------
function throwOrAssignApplicationError( e, eCaught,  applicationErrorSource, applicationErrorId)
{
    webLogMessage(logLevelVerbose, 'throwOrAssignApplicationError: arguments.length=' + arguments.length);
    

    if (arguments.length<4)
    {
        webLogMessage(logLevelInformation, 'throwOrAssignApplicationError: arguments.length <4: internal error');

        var internalError = new Error();
        assignApplicationError(internalError, 0, 0, 'throwOrAssignApplicationError expects 4 or more arguments, received ' + arguments.length, 'planning11.js::throwOrAssignApplicationError', 'errorMissingArguments');
        throw internalError;
     }

    // convert any variadic arguments into a parameter array (allows parameters to be passed between functions more conveniently)
    var applicationErrorParameters = new Array;
    for(var ix=4; ix<arguments.length; ix++)
    {
        applicationErrorParameters[ix-4]=arguments[ix];     
    }

    vThrowOrAssignApplicationError(e, eCaught, applicationErrorSource, applicationErrorId, applicationErrorParameters);
}




// -<>---------------------------------------------------------------------------------------------
// Function: errorSetApplicationPhaseId
// Purpose:  
// Comment:  Intended for the higher level catch block; to indicate the bigger picture of what
//           the application is trying to do when the exception occurred
//
//           (that's why it is broken out of throwOrAssignApplicationError; so it is known that the intention
//            when calling this method is to set the applicationPhase)
//
//           There is no throwApplicationErrorSetPhaseId, it can easily be done be the caller of this method
//
// -><---------------------------------------------------------------------------------------------
function errorSetApplicationPhaseId(e, eCaught, applicationPhaseId, applicationErrorSource, applicationErrorId)
{
    webLogMessage(logLevelInformation, 'errorSetApplicationPhaseId applicationPhaseId=' + applicationPhaseId + ' applicationErrorId='+ applicationErrorId );
    var applicationErrorParameters = new Array;
    for(var ix=5; ix<arguments.length; ix++)
    {
        applicationErrorParameters[ix-5]=arguments[ix];     
    }

    // attach a new error to record this in the chain to the bottom of the list
    vThrowOrAssignApplicationError(e, eCaught, applicationErrorSource, applicationErrorId, applicationErrorParameters);
  
    // set the phase in the error at the top of the list (in principle applies to all errors in this phase: could be better)
    // (also is overwritting if this has already been set so does not currently support chaining application phases)
    if (typeof(applicationPhaseId=='string'))
    {
        e.applicationPhaseId = applicationPhaseId;
    }
    else
    {
        e.applicationErrorId = 'errorApplicationPhaseNotSpecified';
        e.applicationPhaseId = 'phaseErrorHandler';            
    }

    return;
}




// -<>---------------------------------------------------------------------------------------------
// Function: errorNested
// Purpose:  
// Comment:  return fixed error.  Decided if this throws an error the whole thing has crashed
// -><---------------------------------------------------------------------------------------------
function errorNested()
{
    webLogMessage(logLevelInformation, 'errorNested');

    return 'nested error in error handling';
}

// -<>---------------------------------------------------------------------------------------------
// Function: errorNestedXML
// Purpose:  
// Comment:  return fixed nested error XML.  Decided if this throws an error the whole thing has crashed
// -><---------------------------------------------------------------------------------------------
function errorNestedXML(e)
{
    var xmlBody = '';
    xmlBody += '<errorInformation>';
    xmlBody += '<applicationPhaseId>phaseWriteXMLErrorResponse</applicationPhaseId>';
    xmlBody += '<applicationErrorSource>planning11.js::writeXMLErrorResponse:catch(e)</applicationErrorSource>';
    xmlBody += '<applicationErrorId>errorNestedError</applicationErrorId>';
    xmlBody += '<applicationErrorParameters></applicationErrorParameters>';
    xmlBody += '<componentErrorNumber><hex>0xFFFFFFFF</hex><decimal>-1</decimal></componentErrorNumber>';
    xmlBody += '<componentErrorDescription>';
    if (typeof(e)=='object')
    {
        if (typeof(e.description) == 'string' )
        {
            xmlBody += e.description;
        }
    }
     
    xmlBody += '</componentErrorDescription>';
    xmlBody += '</errorInformation>';
    return xmlBody;
}






// -<>---------------------------------------------------------------------------------------------
//     Name: errorDefaultStrings
//  Purpose: initialise error with default strings for display
//  Comment: this means that errorFormat will work even in the case where string lookup etc fails
// -><---------------------------------------------------------------------------------------------
function errorDefaultStrings(e)
{
    e.applicationErrorIdLookupCaption = '';                               
    e.applicationErrorLookupTitle     = '';                               
    e.applicationPhaseIdLookupCaption = '('+ e.applicationPhaseId + ')';  
    e.scriptingMustBeEnabledCaption   = '(errorScriptingMustBeEnabled)';
    e.thisPageErrorDescription='';
    e.furtherInformationCaption='(Further Information)';
    e.serverErrorLabelCaption = '(The server reported the following error:)';
    return;
}

// -<>---------------------------------------------------------------------------------------------
//     Name: errorLookupStrings
//  Purpose: initialise error with default strings for display
//  Comment: this means that errorFormat will work even in the case where string lookup etc fails
// -><---------------------------------------------------------------------------------------------

function errorLookupStrings(e, root)
{
    var xNodeErrorDefinitions = root;
    if(!xNodeErrorDefinitions) 
    { 
         e.thisPageErrorDescription= 'Error definitions XML: root node is NULL<br>';
         return; 
    }   


    var xNodeErrorGenericTitle = xNodeErrorDefinitions.selectSingleNode('genericTitle')
    if (!xNodeErrorGenericTitle) 
    {
        e.thisPageErrorDescription+='cannot find error definitions \'genericTitle\' <br>';
    } 
    else
    {
        e.applicationErrorLookupTitle = xNodeErrorGenericTitle.text;
    }


    var xNodeFurtherInformationCaption = xNodeErrorDefinitions.selectSingleNode('furtherInformationCaption')
    if (!xNodeFurtherInformationCaption) 
    {
        e.thisPageErrorDescription+='cannot find error definitions \'furtherInformationCaption\'<br> ';
    }
    else
    { 
        e.furtherInformationCaption =  xNodeFurtherInformationCaption.text;
    }


    var xNodePhase = xNodeErrorDefinitions.selectSingleNode('phases/phase[@id=\'' + e.applicationPhaseId + '\']'); 
    if (!xNodePhase ) 
    {
        e.thisPageErrorDescription +='cannot find phase description : phase[@id=: "' +   e.applicationPhaseId + '"]'+'<br>';
    } 
    else
    {    
        e.applicationPhaseIdLookupCaption = xNodePhase.text;
    }


    var xNodeError = xNodeErrorDefinitions.selectSingleNode('errors/error[@id=\'' + e.applicationErrorId + '\']'); 
    if (!xNodeError) 
    {
        e.thisPageErrorDescription +='cannot find error definition id: ' + e.applicationErrorId+'<br>';
    } 
    else
    {
        e.applicationErrorIdLookupCaption=xNodeError.text;
    }

    return;
}



// -<>---------------------------------------------------------------------------------------------
//     Name: errorFormatStrings
//  Purpose:
//  Comment: 
// -><---------------------------------------------------------------------------------------------
function errorFormatStrings(e)
{
    webLogMessage(logLevelInformation, 'errorFormatStrings');

    e.applicationErrorLookupTitle =new StringFormatter(e.applicationErrorLookupTitle, e.applicationPhaseIdLookupCaption).value;
    e.applicationErrorIdLookupCaption = new VStringFormatter(e.applicationErrorIdLookupCaption, e.applicationErrorParameters ).value;

 
}






// -<>---------------------------------------------------------------------------------------------
//     Name: errorFormatOutputTable
//  Purpose:
//  Comment: (server side implementation of errorFormatOutput)
//          Error dispay: if a string is untranslated it is put in (brackets), or possibly empty string ''            
// -><---------------------------------------------------------------------------------------------
function errorFormatOutputTable(e)
{
    // to do: put try catch around this too.  
  
    var s='';

    s += '<table  cellspacing="0" cellpadding="0" width="80%" height="80%" border="0" align="center" >';
    s += '<tr>';
    s += '<td width=100% colspan="3" valign="center" align="center">' + e.applicationPhaseIdLookupCaption + '</td>';
    s += '</tr>';

    if (e.serverReturnedError)
    {
        s += '<tr>';
        s += '<td colspan="3" align="center" style="font:8pt/11pt verdana; color:000000">';
        s += e.serverErrorLabelCaption + '</td>';
        s += '</tr>';
    }


    s += '<tr>';
    s += '<td colspan="3" align="center" style="font:8pt/11pt verdana; color:000000">';
    s += '(' + e.applicationErrorId + ')<br><span id="errorCaption">' + e.applicationErrorIdLookupCaption + '</span></td>';
    s += '</tr>';

    s += '<tr height="20%">';
    s += '<td colspan="3">&nbsp;</td>';
    s += '</tr>';
       
 
    // can only display further information if know the browser is IE5.5 and above and something to display
    if (e.browserKnownGood && (e.applicationErrorSource.length || e.description.length || e.thisPageErrorDescription.length))
    {
        s += '<tr height="3">';
        s += '<td colspan="3"><hr color="#C0C0C0" noshade></td>';
        s += '</tr>';

        s += '<tr height="4">';
        s += '<td style="font:8pt/11pt verdana; color:000000" colspan="3" align="right" valign="top">';
        s += '<span id="furtherInformation" ';
        s += 'onmouseover="this.style.cursor=\'hand\';" ';
        s += 'onclick="return furtherInformationClick();" ';
        s += '>' + e.furtherInformationCaption; 
        s += '<img id="furtherInformationButton" border=0 width=16 height=16 src="images/detailsbuttons/details_down_off.gif" >';
        s += '</span>';
        s += '</td>';
        s += '</tr>';

        s += '<tr id="furtherInformationText" style="visibility:hidden" >';
        s += '<td colspan="3" align="right" valign="top">';
        s += '<table width="100%" border="0">';
        s += '<tr>';
        s += '<td width="50%"></td>';
        s += '<td width="50%" align="right">';
        s += '<table cols="2" cellspacing="0" cellpadding="0" border="0" style="font:8pt/11pt verdana; color:000000">';
        if (e.description.length)
        {
            var hexCode = e.number;

            hexCode= hexCode & 0xFFFFFFFF;                       // necessary in IE (v6.0) not required in IIS (W2KProfessional)
            if (hexCode<0) 
            { 
                hexCode  = hexCode + 0x100000000; 
            }

            s += '<tr><td>Component error:</td></tr><tr><td>(0x' + hexCode.toString(16) + ')</td></tr>';
            s += '<tr><td>Component description:</td></tr><tr><td>' + e.description + '</td></tr>';
        }

        if (e.applicationErrorSource.length)
        {
            s += '<tr><td>Script catching error:</td></tr><tr><td>' + e.applicationErrorSource + '</td></tr>';
        }
 
        if (e.thisPageErrorDescription.length)
        {
            s += '<tr><td>Loading this page:</td></tr>';
            s += '<tr><td>' + e.thisPageErrorDescription + '</td></tr>';
        }

        if ( (typeof(e.allOtherErrors)=='object') && (typeof(e.allOtherErrors.length)=='number') )
        {   
            if (e.allOtherErrors.length>0)
            { 
                s+='<tr><td><hr></td></tr><tr><td>All other errors/error stack:</td></tr>';
            }

            for(var ie=0; ie<e.allOtherErrors.length; ie++)
            {
                var err = e.allOtherErrors[ie];
                s +='<tr><td>(' + err.applicationErrorId + ')&nbsp;'+ err.applicationErrorSource +'</td></tr>';     // this will do for now
            }
        }
        else
        {      
            s += '<tr><td>allOtherErrors undefined</td></tr>';
        }

        s += '</table>';

        s += '</td>';
        s += '</tr>';
        s += '</table>';

        s += '</td>';
        s += '</tr>';
    }
    
    s += '</table>';

    return s;
}



// -<>---------------------------------------------------------------------------------------------
// Function: StringFormatter
// Purpose: 
// Comment:  A class that substitutes numbered (from 1) parameters into a format string %1:anything%
// -><---------------------------------------------------------------------------------------------
function StringFormatter(formatString)
{
    webLogMessage(logLevelInformation, '...StringFormatter formatString=' + formatString);  

    var s=formatString;

    for (var x=1; x<arguments.length; x++)
    {
	//alert (arguments[x]);

        re = new RegExp( '%'+x+':[^%]*%', 'g');
	s = s.replace(re, arguments[x]);
    }

    this.value=s;
}


function VStringFormatter(formatString, parameters)
{
    var s=formatString;

    if (parameters)
    {
        webLogMessage(logLevelInformation, '...VStringFormatter parameters.length=' + parameters.length);      

        for (var x=0; x< parameters.length; x++)
        {
            re = new RegExp( '%'+(x+1)+':[^%]*%', 'g');
            s = s.replace(re, parameters[x]);
        }
    }
    this.value=s;
}





// Utility functions

// -<>---------------------------------------------------------------------------------------------
//     Name: checkType
//  Purpose: 
//  Comment: throw or assign error object if value is not the expected type
// 
//  Could have been more fancy here and evaluated the property value from passed description, but
//  decided to keep it simpler
// -><---------------------------------------------------------------------------------------------
function checkType(e, v, description, t, defaultValue)
{
    webLogMessage(logLevelInformation, 'checkType for ' + description);

    if (typeof(t)!='string' )
    {
        throwOrAssignApplicationError(e, new Error(0, 'Error reading:' + description + ' type of type identifier must be string' ), 'planning11.js::safePropertyRead' , 'errorArgumentTypeUnexpected', typeof(t),  'string' ); 
    }

    if (typeof(v)!=t )
    {
        throwOrAssignApplicationError(e, new Error(0, 'Error reading:' + description), 'planning11.js::safePropertyRead' , 'errorValueTypeUnexpected', typeof(v), t ); 
        v = defaultValue;
    }

    return v;
}


// -<>---------------------------------------------------------------------------------------------
//     Name: safeSelectSingleNode
//  Purpose: 
//  Comment: throw error if selection empty
// -><---------------------------------------------------------------------------------------------
function safeSelectSingleNode(e, node, query)
{
    webLogMessage(logLevelInformation, '...safeSelectSingleNode');

    if (!node)
    {
        throwOrAssignApplicationError(e, new Error(0, 'query=(' + query + ')' ), 'planning11.js::safeSelectSingleNode' , 'errorXMLNodeNull' );
    }

    var selectedNode = node.selectSingleNode(query);
    if (!selectedNode)
    {
        throwOrAssignApplicationError(e, new Error(0, 'query=(' + query + ')' ), 'planning11.js::safeSelectSingleNode' , 'errorXMLNodeNotFound', '(' + query + ')');

        // fake up an object with an xml and text properties that is used by caller, when passing in an Error to return
        selectedNode = new Object;
        selectedNode.xml = '';
        selectedNode.text = '';
        selectedNode.notFound= true;
    }

    return selectedNode;
}

// -<>---------------------------------------------------------------------------------------------
//     Name: stringEscape
//  Purpose: reasonably lightweight escape that replace single and double quotes with \' and \"
//  Comment: also replace \ with \\ because sometimes the \ followed by the next character is interpretted
//           ...the main example being 'does not work with usernames begining with x  (\x means hex number follows)
// -><---------------------------------------------------------------------------------------------
function stringEscape(str)
{
   if (typeof(str)=='undefined' )
   {
      str='undefined';
   }
   else
   {

      str   =   str.split('\\').join('\\\\');
      str   =   str.split('\"').join('\\\"');
   }

   return str.split('\'').join('\\\'');
}

// -<>---------------------------------------------------------------------------------------------
// Function: compareHR
// Purpose:
// Comment:  Javascript on some implementations holds numbers as true 32 bit, other implementations
//           only seem to set the high order bits when the number is large enough. Also treatment
//           of the sign extension is differnt.  The following comparision seems to always work though
// -><---------------------------------------------------------------------------------------------
function compareHR(hr, hrConstant)
{
    webLogMessage(logLevelVerbose, '......compareHR');

    var hexCode = hr & 0xFFFFFFFF;
    if (hexCode<0)
    {
        hexCode = 0x100000000 + hexCode;
    }

    return (hexCode==hrConstant);
}
